/* 
 * (c) 2015 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _DIBUJA_H_
#define _DIBUJA_H_

#include <gtk/gtk.h>
#include "simple-canvas.h"
#include "draw-funcs.h"

typedef enum{
	CR_WIDGET,
	CR_SURFACE
}CR_TYPE;

cairo_t *dibuja_get_cr(CR_TYPE type);
GdkPixbuf *dibuja_copy_drawing(void);
cairo_surface_t *dibuja_copy_from_surface(GdkRectangle *rect);
GtkWidget *dibuja_create_canvas(GtkBuilder *builder, int w, int h);
int dibuja_set_filename(char *filename, char *format);
void dibuja_set_title(void);
char *dibuja_get_filename(void);
char *dibuja_get_file_format(void);
GtkWidget *dibuja_get_canvas(void);
cairo_surface_t *dibuja_get_surface(void);
void dibuja_surface_replace(cairo_surface_t *replacement);
GdkPixbuf *dibuja_copy_drawing(void);
void dibuja_rotate_image(double angle);
double dibuja_get_zoom(void);
void dibuja_do_undo(void);
void dibuja_do_redo(void);
cairo_surface_t *dibuja_copy_from_surface(GdkRectangle *rect);
void dibuja_reset_canvas(void);
int dibuja_new_file(char *filename, char *format, int width, int height);
int dibuja_new_file_from_pixbuf(GdkPixbuf *pixbuf, char *filename, char *format);
void dibuja_set_zoom(int zoom_val);
int dibuja_resize(cairo_surface_t *new_surf, char *desc);
int dibuja_get_width(void);
int dibuja_get_height(void);
void dibuja_flip(int dir);
void dibuja_resize_image(gint wnew, gint hnew, gboolean centered);
void dibuja_scale_image(gint wnew, gint hnew);
gboolean dibuja_get_use_trans_sel(void);
gboolean dibuja_tool_cancel(void);

void dibuja_do_grayscale(void);
void dibuja_do_invert(void);
void dibuja_do_pixelize(void);
void dibuja_do_emboss(void);

void dibuja_run_preferences_dialog(void);
void dibuja_clear_image(void);

void dibuja_set_fg_color(GdkColor *color, guint16 alpha);
void dibuja_set_bg_color(GdkColor *color, guint16 alpha);
void dibuja_get_fg_color(GdkColor *color);
void dibuja_get_bg_color(GdkColor *color);
void dibuja_set_line_width(int lw);
int dibuja_get_line_width(void);
void dibuja_get_fg_sccolor(SCColor *color);
void dibuja_get_bg_sccolor(SCColor *color);
void dibuja_set_antialias(gboolean aa);
ShapeFillType dibuja_get_fill_mode(void);
void dibuja_set_fill_mode(ShapeFillType mode);
void dibuja_get_visible_pos(GtkScrolledWindow *scroll, int *x, int *y);

gboolean dibuja_start_take_screenshot(gpointer user_data);
void dibuja_run_fx_dilalog(char *op, char *desc);
void dibuja_do_insta_brannan(char *op);
void dibuja_do_insta_1977(char *op);

void dibuja_show_sel_dims(double w, double h);
void canvas_image_changed(SimpleCanvas *canvas, gpointer data);

void dibuja_file_info_dialog(void);
void dibuja_set_pixbuf_format(GdkPixbufFormat *format);
GdkPixbufFormat *dibuja_get_pixbuf_format(void);


#endif


