/* 
 * (c) 2015 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _DRAW_LIB_H_
	#define _DRAW_LIB_H_

#include <gtk/gtk.h>
#include "simple-canvas.h"

#define DEF_SURF_FORMAT (CAIRO_FORMAT_ARGB32)
#define col2d(c)	((double)((double)(c)/(double)(0xFFFF)))

#define DEG2RAD(d) ((M_PI)/(180.) *((double)d))
#define RAD2DEG(r) ((180.)/(M_PI) *((double)r))

#define FLIP_HORZ	(1)
#define FLIP_VERT	(2)

typedef struct{
	double x1;
	double y1;
	double x2;
	double y2;
}PencilExtents;

typedef enum{
	SHAPE_FILL_NONE = 0,
	SHAPE_FILL_FORE,
	SHAPE_FILL_BACK,
	SHAPE_FILL_LAST
}ShapeFillType;


GdkPoint calc_rotate(double angle, int x, int y);
void draw_line(cairo_t *cr, gdouble x1, gdouble y1, gdouble x2, gdouble y2);
void draw_rectangle(cairo_t *cr, GdkRectangle r, SCColor *fg, SCColor *bg);
void draw_ellipse(cairo_t *cr, GdkRectangle r, SCColor *fg, SCColor *bg);
void draw_rounded_rectangle(cairo_t *cr, GdkRectangle r, SCColor *fg, SCColor *bg);
cairo_surface_t *draw_copy_area(cairo_surface_t *source, int x, int y, int w, int h);
void draw_rotate_pts(double angle_deg, GdkPoint *p1, GdkPoint *p2,
									   GdkPoint *p3, GdkPoint *p4);
GdkPoint draw_calc_rotate(double angle_deg, int x, int y);
void draw_rotate_rectangle(double angle_deg, GdkRectangle *r);

void draw_transform(cairo_t *dest_cr, cairo_surface_t *source,
							double xtr, double ytr, double xsc, double ysc,
							double x, double y);
cairo_surface_t *draw_surface_flip(cairo_surface_t *source, int dir);
cairo_surface_t *draw_rotate_image(cairo_surface_t *source, double angle);
cairo_surface_t *draw_scale_image(cairo_surface_t *source,
								  gint wnew, gint hnew);
cairo_surface_t *draw_resize_image(cairo_surface_t *source,
								   gint wnew, gint hnew,
								   gboolean centered);

void draw_constrain_pt(double p1x, double p1y, double p2x,
							  double p2y, double *xret, double *yret);

void rect_from_pts(GdkRectangle *rectangle, int p1x, int p1y, int p2x, int p2y);
void resize_rect(GdkRectangle *rectangle, int x, int y);
void draw_arrow(cairo_t *cr, double sh_startx, double sh_starty,
				double sh_endx, double sh_endy, GdkRectangle *r);
void draw_rotate_point(double sx, double sy, double *ex, double *ey, double angle);
double draw_line_len(double sx, double sy, double ex, double ey);
double calc_angle(double p1x, double p1y, double p2x, double p2y);


#endif



