/* 
 * (c) 2015 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gtk/gtk.h>

#ifndef __GEGL_VERSION_H__
  #include <gegl-version.h>
#endif

#if GEGL_MAJOR_VERSION == 0 && GEGL_MINOR_VERSION == 1
  #include <gegl-0.1/gegl.h>
#elif GEGL_MAJOR_VERSION == 0 && GEGL_MINOR_VERSION == 2
  #include <gegl-0.2/gegl.h>
extern const gchar * gegl_operation_get_key(const gchar *operation_type,
											const gchar *key_name);
#elif GEGL_MAJOR_VERSION == 0 && GEGL_MINOR_VERSION >= 3
  #if GEGL_MINOR_VERSION == 3
    #include <gegl-0.3/gegl.h>
  #elif GEGL_MINOR_VERSION == 4
    #include <gegl-0.4/gegl.h>
  #endif

  typedef enum{
      GEGL_DIBUJA_INSTA_CURVE_PRESET_NONE = 0,
      GEGL_DIBUJA_INSTA_CURVE_PRESET_1977,
      GEGL_DIBUJA_INSTA_CURVE_PRESET_BRANNAN,
      GEGL_DIBUJA_INSTA_CURVE_PRESET_GOTHAM,
      GEGL_DIBUJA_INSTA_CURVE_PRESET_NASHVILLE
  }DibujaInstaCurve;
#endif

#include <babl/babl.h>

#ifndef GEGL_IS_NODE
	#define GEGL_IS_NODE(n) ((n)!=(NULL))
#endif

#ifndef GEGL_IS_BUFFER
	#define GEGL_IS_BUFFER(b) ((b)!=(NULL))
#endif

GeglBuffer *gegl_buffer_from_surface (cairo_surface_t *ARGB32_surface);
GeglBuffer *fx_copy_buffer(GeglBuffer *src);
cairo_surface_t *buffer_to_surface(GeglBuffer *buffer);

void fx_apply_operation (GeglBuffer *src_buffer, GeglNode *operation,
					GeglBuffer *dest_buffer, const GeglRectangle *dest_rect);

void fx_apply_grey (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
			   const GeglRectangle *dest_rect);

void fx_apply_gaussian_blur (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
						const GeglRectangle *dest_rect,
						gdouble std_dev_x, gdouble std_dev_y);


void fx_apply_invert (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
					  const GeglRectangle *dest_rect);

void fx_apply_brightness_contrast (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
								   const GeglRectangle *dest_rect, 
								   gdouble contrast, gdouble brightness);
void
fx_apply_pixelize (GeglBuffer *src_buffer,
                 GeglBuffer *dest_buffer,
                 const GeglRectangle *dest_rect,
                 int size_x, int size_y);

void fx_apply_insta_1977 (char *op, GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
                 const GeglRectangle *dest_rect);

void fx_apply_insta_brannan (char *op, GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
                 const GeglRectangle *dest_rect);

void fx_apply_emboss(GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
					 const GeglRectangle *dest_rect);




