/* 
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "tools.h"
#include "pencil.h"
#include "dibuja.h"
#include "draw-funcs.h"
#include "undo.h"

#define PENCIL_WIDTH	(1)
#define ALIAS			(2)

static gboolean pencil_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data);
static gboolean pencil_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data);
static gboolean pencil_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data);

static void pencil_draw_pencil(cairo_t *cr, double sx, double sy, double ex, double ey);

static DrawTool tool = {
				pencil_button_press,
				pencil_motion,
				pencil_button_release,
				NULL,//pencil_expose,
				NULL,
				NULL
			};

static GdkRectangle undo_area = {0};
static cairo_surface_t *tmp_surface = NULL;
static double sx, sy, ex, ey;

DrawTool *pencil_get_tool(void)
{
	return &tool;
}

static void pencil_calc_update_area(GdkRectangle *r, double x1, double y1)
{
	double x, y;

	if(r){
		x = MAX(x1, r->x + r->width);
		y = MAX(y1, r->y + r->height);

		r->x = MIN(x1, r->x);
		r->y = MIN(y1, r->y);

		r->width = ABS(x - r->x);
		r->height = ABS(y - r->y);
	}
}
static void pencil_draw_pencil(cairo_t *cr, double x1, double y1, double x2, double y2)
{
	cairo_set_line_width(cr, 1.);

	draw_line(cr, x1, y1, x2, y2);
}

static gboolean pencil_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if(1 == event->button){
		GdkRectangle copy = {0};

		copy.x = copy.y = 0;
		copy.width = cairo_image_surface_get_width(dibuja_get_surface());
		copy.height = cairo_image_surface_get_height(dibuja_get_surface());
		tmp_surface = dibuja_copy_from_surface(&copy);

		sx = ex = event->x;
		sy = ey = event->y;
		rect_from_pts(&undo_area, sx, sy, ex, ey);
	}

	return TRUE;
}


static gboolean pencil_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
	if (event->state & GDK_BUTTON1_MASK){
		GdkRectangle old, cur;
		int extra = PENCIL_WIDTH + ALIAS;
		cairo_t *cr;

		rect_from_pts(&old, sx, sy, ex, ey);

		ex = event->x;
		ey = event->y;

		pencil_calc_update_area(&undo_area, ex, ey);

		cr = dibuja_get_cr(CR_SURFACE);
		pencil_draw_pencil(cr, sx, sy, ex, ey);
		cairo_destroy(cr);

		rect_from_pts(&cur, sx, sy, ex, ey);

		sx = ex;
		sy = ey;

		gdk_rectangle_union(&old, &cur, &cur);

		resize_rect(&cur, extra, extra);

		simple_canvas_update_rect(SIMPLE_CANVAS(widget), &cur);
	}

	return TRUE;
}

static gboolean pencil_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if(1 == event->button){
		cairo_t *cr;
		cairo_surface_t *copy_from_tmp = NULL;
		int extra = PENCIL_WIDTH + ALIAS;

		ex = event->x;
		ey = event->y;

		cr = dibuja_get_cr(CR_SURFACE);
		pencil_draw_pencil(cr, sx, sy, ex, ey);

		cairo_destroy(cr);

		pencil_calc_update_area(&undo_area, ex, ey);
		resize_rect(&undo_area, extra, extra);

		copy_from_tmp = draw_copy_area(tmp_surface, undo_area.x, undo_area.y,
									   undo_area.width, undo_area.height);
		undo_add_from_surface(SIMPLE_CANVAS(widget), copy_from_tmp,
													 undo_area, _("Pencil"));
			
		gtk_widget_queue_draw(widget);
	}

	if(tmp_surface){
		cairo_surface_destroy(tmp_surface);
		tmp_surface = NULL;
	}

	return TRUE;
}



