/*
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Minimal printing module...
 * If the image does not fit in the paper
 * it is cropped
 */
/* TODO: Better printing */

#include <gtk/gtk.h>
#include "dibuja.h"
#include "draw-funcs.h"

static GtkPrintSettings *settings = NULL;
static GtkPageSetup *page_setup = NULL;

static void cb_begin_print(GtkPrintOperation *op,
						   GtkPrintContext *ctx,
						   gpointer data)
{
	gtk_print_operation_set_n_pages(op, 1);
}

static void cb_draw_page(GtkPrintOperation *op,
						 GtkPrintContext *ctx,
						 gint page_nr,
						 gpointer data)
{
	cairo_surface_t *surface;
	cairo_t *cr;
	gint neww, newh;
	gint orgw, orgh;
	gint ctxw, ctxh;

	ctxw = gtk_print_context_get_width (ctx);
	ctxh = gtk_print_context_get_height (ctx);

	orgw = dibuja_get_width();
	orgh = dibuja_get_height();

	if(orgw > ctxw || orgh > ctxh){
		if(orgh > orgw){
			newh = ctxh;
			neww = newh * orgw / orgh + .5;
		}
		else{
			neww = ctxw;
			newh = neww * orgh / orgw + .5;
		}
		surface = draw_scale_image(dibuja_get_surface(), neww, newh);
	}
	else{
		 neww = orgw;
		 newh = orgh;
		 surface = draw_copy_area(dibuja_get_surface(), 0, 0, 
										  dibuja_get_width(),
										  dibuja_get_height());
	}
	cr = gtk_print_context_get_cairo_context (ctx);
	cairo_translate(cr, ctxw / 2 - neww / 2., 0.);
	cairo_set_source_surface(cr, surface, 0., 0.);
	cairo_paint(cr);

	cairo_surface_destroy(surface);
}

void start_printing(GtkWidget *widget, gpointer data)
{
	GtkPrintOperation *po;

	po = gtk_print_operation_new();
	if(GTK_IS_PRINT_OPERATION(po)){
		g_signal_connect (po, "begin_print", G_CALLBACK (cb_begin_print), NULL);
		g_signal_connect (po, "draw_page", G_CALLBACK (cb_draw_page), NULL);

		if(page_setup){
			gtk_print_operation_set_default_page_setup(po, page_setup);
		}
		gtk_print_operation_run(po, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
								GTK_WINDOW (gtk_widget_get_toplevel(widget)),
									   NULL);

		g_object_unref(po);
	}
}

void printing_page_setup(GtkWidget *menu, gpointer data)
{
	GtkPageSetup *new_page_setup;
	GtkWindow *window = GTK_WINDOW (gtk_widget_get_toplevel(menu));

	if(settings == NULL){
		settings = gtk_print_settings_new();
		gtk_print_settings_unset(settings, GTK_PRINT_SETTINGS_SCALE);
		g_object_set_data_full(G_OBJECT(dibuja_get_canvas()), "print-settings",
							   (gpointer)settings, g_object_unref);
	}

	new_page_setup = gtk_print_run_page_setup_dialog(window, page_setup, settings);
	if(page_setup)
		g_object_unref(page_setup);

	page_setup = new_page_setup;
	/* Destroys last pagesetup on exit */
	g_object_set_data_full(G_OBJECT(dibuja_get_canvas()), "page-setup",
							   (gpointer)page_setup, g_object_unref);
}

