/* 
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "resize.h"
#include "dibuja.h"
#include "util.h"
#include "selection.h"

static GtkWidget *resize_dialog = NULL;

gboolean entry_get_num(char *name, int *num)
{
	GtkEntry *entry = GTK_ENTRY(get_widget(name));
	gchar *text = NULL;

	if(GTK_IS_ENTRY(entry) && num){
		text = g_strdup(gtk_entry_get_text(entry));
		if(text){
			*num = (int)g_ascii_strtod(text, NULL);
			g_free(text);
			return TRUE;
		}
	}

	return FALSE;
}

void dialog_resize_button_apply_clicked(void)
{
	gint w, h;

	if(entry_get_num("dialog_resize_entry_width", &w)){
		if(entry_get_num("dialog_resize_entry_height", &h)){
			GtkWidget *ch = get_widget("dialog_resize_checkbutton");
			gboolean checked;

			checked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ch));

			if(sel_have()){
				sel_resize_cur(w, h, checked);
			}
			else{
				dibuja_resize_image(w, h, checked);
			}
		}
	}
}

void dialog_resize_run(void)
{
	gint ret = 0;
	gchar *text = NULL;
	int w, h;
	GtkWidget *widget;

	if(!GTK_IS_WIDGET(resize_dialog)){
		resize_dialog = get_widget("dialog_resize");
	}

	if(sel_have()){
		cairo_surface_t *surf = sel_get_cur_image();
		w = cairo_image_surface_get_width(surf);
		h = cairo_image_surface_get_height(surf);
	}
	else{
		w = dibuja_get_width();
		h = dibuja_get_height();
	}

	text = g_strdup_printf(_("<b>Current size: %d x %d</b>"), w, h);
	if(text){
		widget = get_widget("dialog_resize_label_size");

		gtk_label_set_markup(GTK_LABEL(widget), text);
		g_free(text);
	}

	text = g_strdup_printf("%d", w);
	if(text){
		widget = get_widget("dialog_resize_entry_width");
		gtk_entry_set_text(GTK_ENTRY(widget), text);
		g_free(text);
	}
	text = g_strdup_printf("%d", h);
	if(text){
		widget = get_widget("dialog_resize_entry_height");
		gtk_entry_set_text(GTK_ENTRY(widget), text);
		g_free(text);
	}

	ret = gtk_dialog_run (GTK_DIALOG (resize_dialog));
	switch (ret){
    	case GTK_RESPONSE_APPLY:
    		dialog_resize_button_apply_clicked();
    		break;
    	default:
    		break;
	}

	gtk_widget_hide (resize_dialog);
}
