/* 
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "rotate.h"
#include "dibuja.h"
#include "util.h"
#include "selection.h"


static GtkWidget *dialog = NULL;
static GtkWidget *darea = NULL;
static double angle = 0.;

void dialog_rotate_button_close_activate(GtkWidget *widget, gpointer data)
{
	
}

void dialog_rotate_button_apply_activate(GtkWidget *widget, gpointer data)
{
	
}

void dialog_rotate_angle_spin_value_changed(GtkSpinButton *spin, gpointer data)
{
	angle = gtk_spin_button_get_value(spin);
	if(GTK_IS_WIDGET(darea) && gtk_widget_get_visible(darea)){
		gtk_widget_queue_draw(darea);
	}
}
// TODO: Write preview widget
gboolean dialog_rotate_darea_expose_event_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t *cr, *ct;
	cairo_surface_t *surf, *tmp;
	double w, h;
	double scale = 1;


	if(sel_have()){
		surf = sel_get_cur_image();
		w = (double)cairo_image_surface_get_width(surf);
		h = (double)cairo_image_surface_get_height(surf);
	}
	else{
		surf = dibuja_get_surface();
		w = (double)dibuja_get_width();
		h = (double)dibuja_get_height();
	}
	scale = MIN(widget->allocation.width, widget->allocation.height) / MAX(w, h);

	tmp = cairo_image_surface_create(DEF_SURF_FORMAT, (int)w * scale, (int)h * scale);
	ct = cairo_create(tmp);
	cairo_scale(ct, scale, scale);
	cairo_set_source_surface(ct, surf, 0., 0.);
	cairo_paint(ct);
	cairo_destroy(ct);

	cr = gdk_cairo_create(widget->window);

	w = (double)cairo_image_surface_get_width(tmp);
	h = (double)cairo_image_surface_get_height(tmp);

	cairo_translate(cr, (double)widget->allocation.width / 2.,
						(double)widget->allocation.height / 2.);
	cairo_rotate(cr, angle * M_PI / 180.);
	cairo_translate(cr, -w / 2.0, -h / 2.0);

	cairo_set_source_surface(cr, tmp, 0, 0);
	cairo_paint(cr);
	cairo_destroy(cr);

	cairo_surface_destroy(tmp);

	return TRUE;
}

void dialog_rotate_run(void)
{
	gint ret = 0;

	if(!GTK_IS_WIDGET(dialog)){
		dialog = get_widget("dialog_rotate");
	}
	if(!GTK_IS_WIDGET(darea)){
		darea = get_widget("dialog_rotate_darea");
	}

	angle = 0.;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("dialog_rotate_angle_spin")), angle);
	ret = gtk_dialog_run (GTK_DIALOG (dialog));
	switch (ret){
    	case GTK_RESPONSE_APPLY:
    		if(sel_have()){
    			sel_rotate(angle);
    		}
    		else{
    			dibuja_rotate_image(angle);
    		}
    		break;
    	default:
    		break;
	}

	gtk_widget_hide (dialog);
}
