/* 
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "dibuja.h"
#include "util.h"
#include "selection.h"

static GtkWidget *scale_dialog = NULL;
static int origw, origh;
static GtkWidget *wspin = NULL, *hspin = NULL;
static gboolean keep_aspect = TRUE;


void dialog_scale_checkbutton_toggled(GtkToggleButton *scale_checkbutton, gpointer user_data)
{
	keep_aspect = gtk_toggle_button_get_active(scale_checkbutton);
}

void dialog_scale_spinbutton_width_value_changed(GtkSpinButton *spin, gpointer data)
{
	double val;
	gboolean focus = 0;

	focus = gtk_widget_has_focus(GTK_WIDGET(spin));
	val = gtk_spin_button_get_value(spin);

	if(focus && keep_aspect)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(hspin), val * origh / origw + .5);
}

void dialog_scale_spinbutton_height_value_changed(GtkSpinButton *spin, gpointer data)
{
	double val;
	gboolean focus = 0;

	focus = gtk_widget_has_focus(GTK_WIDGET(spin));
	val = gtk_spin_button_get_value(spin);

	if(focus && keep_aspect)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(wspin), val * origw / origh + .5);
}

void dialog_scale_run(void)
{
	gint ret = 0;

	GtkWidget *widget;
	char *markup;
	int neww, newh;

	if(!GTK_IS_WIDGET(scale_dialog)){
		scale_dialog = get_widget("dialog_scale");
	}

	if(sel_have()){
		cairo_surface_t *surf = sel_get_cur_image();
		origw = cairo_image_surface_get_width(surf);
		origh = cairo_image_surface_get_height(surf);
	}
	else{
		origw = dibuja_get_width();
		origh = dibuja_get_height();
	}

	widget = get_widget("dialog_scale_label_size");
	markup = g_strdup_printf(_("<b>Current Size: %d x %d</b>"), origw, origh);
	gtk_label_set_markup(GTK_LABEL(widget), markup);
	g_free(markup);

	wspin = get_widget("dialog_scale_spinbutton_width");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(wspin), (double)origw);
	hspin = get_widget("dialog_scale_spinbutton_height");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(hspin), (double)origh);

	ret = gtk_dialog_run (GTK_DIALOG (scale_dialog));
	switch (ret){
    	case GTK_RESPONSE_APPLY:
    		neww = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wspin));
    		newh = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hspin));

    		if(sel_have()){
    			sel_scale(neww, newh);
    		}
    		else{
    			dibuja_scale_image(neww, newh);
    		}
    		break;
    	default:
    		break;
	}

	gtk_widget_hide (scale_dialog);
}

