/*
 * sel-handle.c
 * Copyright (C) Juan Balderas 2012 - 2018 <juan.balderas.0@gmail.com>
 * 
 * sel-handle.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * sel-handle.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SEL_HANDLE_H_
#define _SEL_HANDLE_H_

#include <glib-object.h>
#include <gtk/gtk.h>
#include <math.h>

G_BEGIN_DECLS

#define SEL_TYPE_HANDLE             (sel_handle_get_type ())
#define SEL_HANDLE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), SEL_TYPE_HANDLE, SelHandle))
#define SEL_HANDLE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), SEL_TYPE_HANDLE, SelHandleClass))
#define SEL_IS_HANDLE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SEL_TYPE_HANDLE))
#define SEL_IS_HANDLE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), SEL_TYPE_HANDLE))
#define SEL_HANDLE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), SEL_TYPE_HANDLE, SelHandleClass))

#define DEF_HANDLE_WIDTH (8)
#define DEF_HANDLE_HEIGHT DEF_HANDLE_WIDTH

typedef enum{
	SEL_HANDLE_RECT,
	SEL_HANDLE_ROUND
}SelHandleType;


typedef struct _SelHandleClass SelHandleClass;
typedef struct _SelHandle SelHandle;
typedef struct _SelHandlePrivate SelHandlePrivate;

struct _SelHandleClass
{
	GObjectClass parent_class;
};

struct _SelHandle
{
	GObject parent_instance;
	SelHandlePrivate *priv;
};

GType sel_handle_get_type (void) G_GNUC_CONST;
void sel_handle_draw (SelHandle *sh, cairo_t *cr);
void sel_handle_move (SelHandle *sh, gint x, gint y);
SelHandle * sel_handle_new (void);
void sel_handle_set_active (SelHandle *sh, gboolean active);
gboolean sel_handle_is_active (SelHandle *sh);
gboolean sel_handle_pt_in (SelHandle *sh, gint x, gint y);
void sel_handle_get_pos (SelHandle *sh, GdkPoint *pt);
void sel_handle_set_color (SelHandle *sh, GdkColor *color);
void sel_handle_set_type(SelHandle *sh, SelHandleType type);
void sel_handle_set_dims(SelHandle *sh, guint w, guint h);
/* Get offset from inside the handle 
   Returns TRUE if the point is in the handle,
   and x & y are set to the offset */
gboolean sel_handle_get_pt_offset(SelHandle *sh, gint *x, gint *y);
gint sel_handle_get_width (SelHandle *sh);
gint sel_handle_get_height (SelHandle *sh);

G_END_DECLS

#endif /* _SEL_HANDLE_H_ */
