/*
 * sel-obj.h
 * Copyright (C) Juan Balderas 2015 - 2018 <juan.balderas.0@gmail.com>
 * 
 * sel-obj.h is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * sel-obj.h is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SEL_OBJ_H_
#define _SEL_OBJ_H_

#include <glib-object.h>
#include <gtk/gtk.h>
#include <math.h>

G_BEGIN_DECLS

#define SEL_TYPE_OBJ             (sel_obj_get_type ())
#define SEL_OBJ(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), SEL_TYPE_OBJ, SelObj))
#define SEL_OBJ_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), SEL_TYPE_OBJ, SelObjClass))
#define SEL_IS_OBJ(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SEL_TYPE_OBJ))
#define SEL_IS_OBJ_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), SEL_TYPE_OBJ))
#define SEL_OBJ_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), SEL_TYPE_OBJ, SelObjClass))


typedef enum{
	SEL_OBJ_RECT,
	SEL_OBJ_ROUND
}SelObjType;


typedef struct _SelObjClass SelObjClass;
typedef struct _SelObj SelObj;
typedef struct _SelObjPrivate SelObjPrivate;

struct _SelObjClass
{
	GObjectClass parent_class;
};

struct _SelObj
{
	GObject parent_instance;
	SelObjPrivate *priv;
};


GType sel_obj_get_type (void) G_GNUC_CONST;
SelObj *sel_obj_new(void);
//int sel_obj_create_from_rect(SelObj *sel, GdkRectangle *rect);
void sel_obj_set_image(SelObj *sel, cairo_surface_t *new_img);
cairo_surface_t *sel_obj_swap_image(SelObj *sel, cairo_surface_t *new_img);
SelObj *sel_obj_create_from_surface(cairo_surface_t *source);
void sel_obj_move(SelObj *sel, int x, int y, int use_offset);
void sel_obj_resize(SelObj *sel, int width, int height);
cairo_surface_t *sel_obj_get_image(SelObj *sel);
void sel_obj_get_rectangle(SelObj *sel, GdkRectangle *rect);
int sel_obj_point_in(SelObj *sel, int x, int y);
void sel_obj_set_offset(SelObj *sel, int x, int y);
void sel_obj_set_orig_pos(SelObj *sel, GdkRectangle *rect);
void sel_obj_get_orig_pos(SelObj *sel, GdkRectangle *rect);
cairo_surface_t *sel_obj_remove_image(SelObj *sel);
gboolean sel_obj_get_modified(SelObj *sel);
void sel_obj_set_modified(SelObj *sel, gboolean modified);
char *sel_obj_get_description(SelObj *sel);
void sel_obj_set_description(SelObj *sel, char *desc);


G_END_DECLS

#endif /* _SEL_OBJ_H_ */

