/*
 * (c) 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#if 0
  #include <stdlib.h>
  #include <string.h>
#endif
#include "settings.h"
#include "simple-canvas.h"
#include "config.h"

#define GROUP	PACKAGE

#if 0
static void settings_print(Settings *settings, FILE *fp)
{
	g_return_if_fail(settings && fp);

	fprintf(fp, "%s\n", settings->version);
	fprintf(fp, "%d\n", settings->width);
	fprintf(fp, "%d\n", settings->height);
	fprintf(fp, "%s\n", settings->fontname);

	fprintf(fp, "%f %f %f %f\n",
			settings->fgcolor[0],
			settings->fgcolor[1],
			settings->fgcolor[2],
			settings->fgcolor[3]);

	fprintf(fp, "%f %f %f %f\n",
			settings->bgcolor[0],
			settings->bgcolor[1],
			settings->bgcolor[2],
			settings->bgcolor[3]);

	fprintf(fp, "%d\n", settings->line_width);
	fprintf(fp, "%d\n", settings->fill_mode);
	fprintf(fp, "%d", settings->antialias);
}
#endif
int settings_read(Settings *settings, char *filename)
{
	char *p;
	int i;
	gsize n;
	GKeyFile *kf = NULL;
	GError *error = NULL;
	double *d;
	int width  = gdk_screen_width();
	int height = gdk_screen_height();

	if(!filename)
		return 0;

//  Set defaults...
	g_strlcpy(settings->version, PACKAGE_VERSION, VERSION_MAX);
	settings->width = 865;
	settings->height = 530;
	g_strlcpy(settings->fontname, "System 10", FONTNAME_MAX);

	settings->fgcolor[0] = 0.;
	settings->fgcolor[1] = 0.;
	settings->fgcolor[2] = 0.;
	settings->fgcolor[3] = 1.;

	settings->bgcolor[0] = 1.;
	settings->bgcolor[1] = 1.;
	settings->bgcolor[2] = 1.;
	settings->bgcolor[3] = 1.;

	settings->line_width = 1;
	settings->fill_mode = SC_FILL_NONE;
	settings->antialias = 1;
	settings->use_arrow = 0;
	settings->trans_sel = 0;

	kf = g_key_file_new();
	i = g_key_file_load_from_file(kf, filename, G_KEY_FILE_NONE, NULL);
	if(!i){
		if(error){
			g_print("ERROR: %s", error->message);
			g_error_free(error);
		}
	}
	else{

		if((p = g_key_file_get_string(kf, GROUP, "version", NULL)) != NULL){
			g_strlcpy(settings->version, p, VERSION_MAX);
			g_free(p);
		}

		if((i = g_key_file_get_integer(kf, GROUP, "width", NULL)) > 0){
			if(i <= width + 100)
				settings->width = i;
		}

		if((i = g_key_file_get_integer(kf, GROUP, "height", NULL)) > 0){
			if(i <= height + 100)
				settings->height = i;
		}

		if((p = g_key_file_get_string(kf, GROUP, "fontname", NULL)) != NULL){
			if('\0' != p[0]){
				g_strlcpy(settings->fontname, p, FONTNAME_MAX);
			}
			g_free(p);
		}

		if((d = g_key_file_get_double_list(kf, GROUP, "fgcolor", &n, NULL)) != NULL){
			for(i = 0; i < (int)n && i < COL_MAX; i++)
				settings->fgcolor[i] = d[i];

			g_free(d);
		}

		if((d = g_key_file_get_double_list(kf, GROUP, "bgcolor", &n, NULL)) != NULL){
			for(i = 0; i < (int)n && i < COL_MAX; i++)
				settings->bgcolor[i] = d[i];

			g_free(d);
		}

		i = g_key_file_get_integer(kf, GROUP, "line_width", NULL);
		if(i > 0 && i <= 20)
			settings->line_width = i;

		i = g_key_file_get_integer(kf, GROUP, "fill_mode", NULL);
		if(i >= SC_FILL_NONE && i < SC_FILL_LAST){
			settings->fill_mode = i;
		}

		settings->antialias = g_key_file_get_boolean(kf, GROUP,
													 "antialias", NULL);
		settings->use_arrow = g_key_file_get_boolean(kf, GROUP,
													 "use_arrow", NULL);
		settings->trans_sel = g_key_file_get_boolean(kf, GROUP,
													 "trans_sel", NULL);
	}

	if(kf){
		g_key_file_free(kf);
	}

#if 0
	settings_print(settings, stdout);
#endif

	return 1;
}

void settings_write(Settings *settings, char *filename)
{
	GKeyFile *kf;
	char comment[] = " dibuja config file. Do not modify this file directly.";

	g_return_if_fail(settings && filename);

#if 0
	settings_print(settings, stdout);
#endif

	kf = g_key_file_new();
	if(!kf){
		return;
	}

	g_key_file_set_comment     (kf, NULL,  NULL,         comment, NULL);
	g_key_file_set_string      (kf, GROUP, "version",    settings->version);
	g_key_file_set_integer     (kf, GROUP, "width",      settings->width);
	g_key_file_set_integer     (kf, GROUP, "height",     settings->height);
	g_key_file_set_string      (kf, GROUP, "fontname",   settings->fontname);
	g_key_file_set_double_list (kf, GROUP, "fgcolor",    settings->fgcolor, COL_MAX);
	g_key_file_set_double_list (kf, GROUP, "bgcolor",    settings->bgcolor, COL_MAX);
	g_key_file_set_integer     (kf, GROUP, "line_width", settings->line_width);
	g_key_file_set_integer     (kf, GROUP, "fill_mode",  settings->fill_mode);
	g_key_file_set_boolean     (kf, GROUP, "antialias",  settings->antialias);
	g_key_file_set_boolean     (kf, GROUP, "use_arrow",  settings->use_arrow);
	g_key_file_set_boolean     (kf, GROUP, "trans_sel",  settings->trans_sel);

	g_key_file_save_to_file(kf, filename, NULL);
	g_key_file_free(kf);
}

