/* 
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __SIMPLE_CANVAS_H__
#define __SIMPLE_CANVAS_H__


#include <gtk/gtk.h>


G_BEGIN_DECLS

#define SIMPLE_TYPE_CANVAS            (simple_canvas_get_type ())
#define SIMPLE_CANVAS(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SIMPLE_TYPE_CANVAS, SimpleCanvas))
#define SIMPLE_CANVAS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SIMPLE_TYPE_CANVAS, SimpleCanvasClass))
#define SIMPLE_IS_CANVAS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SIMPLE_TYPE_CANVAS))
#define SIMPLE_IS_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SIMPLE_TYPE_CANVAS))
#define SIMPLE_CANVAS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SIMPLE_TYPE_CANVAS, SimpleCanvasClass))


typedef struct _SimpleCanvas         SimpleCanvas;
typedef struct _SimpleCanvasClass    SimpleCanvasClass;
typedef struct _SimpleCanvasPrivate  SimpleCanvasPrivate;

typedef struct{
	double r, g, b, a;
}SCColor;

typedef enum{
	SC_FILL_NONE = 0,
	SC_FILL_FORE,
	SC_FILL_BACK,
	SC_FILL_LAST
}SCFillType;

struct _SimpleCanvas
{
	GtkDrawingArea parent;
	SimpleCanvasPrivate *priv;
};

struct _SimpleCanvasClass
{
	GtkDrawingAreaClass parent_class;

	void (* simple_canvas_expose)(SimpleCanvas *canvas, cairo_t *cr,
													gpointer user_data);
	void (* simple_canvas_button_press)(SimpleCanvas *canvas,
								GdkEventButton *event, gpointer user_data);
	void (* simple_canvas_button_release)(SimpleCanvas *canvas,
								GdkEventButton *event, gpointer user_data);
	void (* simple_canvas_pointer_motion)(SimpleCanvas *canvas,
								GdkEventMotion *event, gpointer user_data);
	void (* simple_canvas_image_changed)(SimpleCanvas *canvas);
};


GType       simple_canvas_get_type        (void) G_GNUC_CONST;
GtkWidget*  simple_canvas_new             (int width, int height);
cairo_t     *simple_canvas_get_context    (SimpleCanvas *canvas);
cairo_t     *simple_canvas_get_window_context(SimpleCanvas *canvas);
void        simple_canvas_get_release_pt  (SimpleCanvas *canvas, GdkPoint *pt);
void        simple_canvas_get_press_pt    (SimpleCanvas *canvas, GdkPoint *pt);
void        simple_canvas_set_image       (SimpleCanvas *canvas,
											cairo_surface_t *surface);
cairo_surface_t *simple_canvas_swap_image (SimpleCanvas *canvas,
											cairo_surface_t *surface);
cairo_surface_t *simple_canvas_get_image  (SimpleCanvas *canvas);
double      simple_canvas_set_zoom        (SimpleCanvas *canvas, double zoom);
double      simple_canvas_get_zoom        (SimpleCanvas *canvas);
void        simple_canvas_set_show_grid   (SimpleCanvas *canvas,
											gboolean show_grid);
void        simple_canvas_set_max_zoom    (SimpleCanvas *canvas, double zoom);

void simple_canvas_update_rect(SimpleCanvas *sc, GdkRectangle *r);

double img_to_win(double n, double zoom);
double win_to_img(double n, double zoom);

int simple_canvas_get_line_width(SimpleCanvas *sc);
void simple_canvas_set_line_width(SimpleCanvas *sc, int line_width);
void simple_canvas_get_fg_color(SimpleCanvas *sc, GdkColor *color);
void simple_canvas_set_fg_color(SimpleCanvas *sc, GdkColor *color);
void simple_canvas_get_bg_color(SimpleCanvas *sc, GdkColor *color);
void simple_canvas_set_bg_color(SimpleCanvas *sc, GdkColor *color);
cairo_antialias_t  simple_canvas_get_antialias(SimpleCanvas *sc);
void simple_canvas_set_antialias(SimpleCanvas *sc, cairo_antialias_t aa);
void simple_canvas_get_fg_rgba(SimpleCanvas *sc, double *r, double *g, double *b, double *a);
void simple_canvas_get_bg_rgba(SimpleCanvas *sc, double *r, double *g, double *b, double *a);
void simple_canvas_set_fg_rgba(SimpleCanvas *sc, double r, double g, double b, double a);
void simple_canvas_set_bg_rgba(SimpleCanvas *sc, double r, double g, double b, double a);


G_END_DECLS

#endif /* __SIMPLE_CANVAS_H__ */
