/* 
 * (c) 2015 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _TOOLS_H_
#define _TOOLS_H_

#include <gtk/gtk.h>

typedef gboolean (*ButtonPress)(GtkWidget *widget, GdkEventButton *event, gpointer data);
typedef gboolean (*Motion)(GtkWidget *widget, GdkEventMotion *event, gpointer data);
typedef gboolean (*ButtonRelease)(GtkWidget *widget, GdkEventButton *event, gpointer data);
typedef gboolean (*Expose)(GtkWidget *widget, cairo_t *cr, gpointer data);
typedef void (*Finish)(void);
typedef gboolean (*Cancel)(void);
typedef gboolean (*PopupMenu)(GtkWidget *widget, GdkEventButton *event, gpointer data);

typedef void (*DrawShape)(cairo_t *cr, gdouble x, gdouble y, gdouble w, gdouble h, int filled);

typedef struct{
	ButtonPress button_press;
	Motion motion;
	ButtonRelease button_release;
	Expose expose;
	Finish finish;
	PopupMenu popup;
	Cancel cancel;
}DrawTool;

typedef enum{
	TOOL_TYPE_LINES = 0,
	TOOL_TYPE_RECTANGLE,
	TOOL_TYPE_ELLIPSE,
	TOOL_TYPE_PENCIL,
	TOOL_TYPE_TEXT,
	TOOL_TYPE_BRUSH,
	TOOL_TYPE_SELECTION,
	TOOL_TYPE_FLOODFILL,
	TOOL_TYPE_ROUNDED_RECTANGLE,
	TOOL_TYPE_POLYLINE,
	TOOL_TYPE_CURVE,
	TOOL_TYPE_LAST
}ToolType;

DrawTool *tool_get(void);
void tool_init(GtkBuilder *builder);
void tools_toggle_sel_button(gboolean is_active);
void tools_toggle_line_button(gboolean is_active);
gboolean tools_get_draw_arrow(void);
void tools_toggle_brush_button(gboolean is_active);

#endif
