/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * undo-linear.c
 * Copyright (C)  2012 - 2018 Juan Balderas
 * 
 * undo-linear.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * undo-linear.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _UNDO_LINEAR_H_
#define _UNDO_LINEAR_H_

//#include <gtk/gtk.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define UNDO_TYPE_LINEAR             (undol_get_type ())
#define UNDO_LINEAR(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), UNDO_TYPE_LINEAR, UndoLinear))
#define UNDO_LINEAR_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), UNDO_TYPE_LINEAR, UndoLinearClass))
#define UNDO_IS_LINEAR(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), UNDO_TYPE_LINEAR))
#define UNDO_IS_LINEAR_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), UNDO_TYPE_LINEAR))
#define UNDO_LINEAR_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), UNDO_TYPE_LINEAR, UndoLinearClass))

#define UNDO_MAX_DEFAULT	(10)

typedef struct _UndoLinearClass UndoLinearClass;
typedef struct _UndoLinear UndoLinear;
typedef struct _UndoLinearPrivate UndoLinearPrivate;

typedef void (*UndoCallback)(UndoLinear *ul, gpointer data);

struct _UndoLinearClass
{
	GObjectClass parent_class;
	void (* undol_undoing)(UndoLinear *ul, gpointer data);
	void (* undol_redoing)(UndoLinear *ul, gpointer data);

	void (* undol_undo_delete)(UndoLinear *ul, gpointer data);
	void (* undol_redo_delete)(UndoLinear *ul, gpointer data);

	void (* undol_undo_max_delete)(UndoLinear *ul, gpointer data);
};

struct _UndoLinear
{
	GObject parent_instance;
	UndoLinearPrivate *priv;
};

GType undol_get_type (void) G_GNUC_CONST;
UndoLinear *undol_new (void);
gboolean undol_add(UndoLinear *ul, gpointer data);
gboolean undol_can_undo(UndoLinear *ul);
gboolean undol_can_redo(UndoLinear *ul);
void undol_set_undo_delete_callback(UndoLinear *ul, UndoCallback undo_delete);
void undol_set_redo_delete_callback(UndoLinear *ul, UndoCallback redo_delete);
gboolean undol_do_undo(UndoLinear *ul);
gboolean undol_do_redo(UndoLinear *ul);
guint undol_get_undo_cnt(UndoLinear *ul);
guint undol_get_undo_max(UndoLinear *ul);
void undol_set_undo_max(UndoLinear *ul, guint max);
void undol_clear(UndoLinear *ul);
gpointer undol_get_undo_data(UndoLinear *ul);
gpointer undol_get_redo_data(UndoLinear *ul);


//GList *get_undo(UndoLinear *ul);
//GList *get_redo(UndoLinear *ul);

G_END_DECLS

#endif /* _UNDO_LINEAR_H_ */

