/* 
 * (c) 2014 - 2018 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "util.h"
#include "dibuja.h"
#include "color.h"
#include "simple-canvas.h"
#include "file.h"

void msg_dlg_forall(GtkWidget *widget, gpointer data);
static GtkWidget *main_window = NULL;
static GtkBuilder *gtk_builder = NULL;

void util_set_tool_page(int tooltype, int page_num)
{
	gtk_notebook_set_current_page(GTK_NOTEBOOK(get_widget("Tool Properties")), page_num);

	switch(page_num){
		case 2:
		case 0:
			if(0 == tooltype){
				gtk_widget_show(get_widget("check_button_use_arrow"));
			}
			else{
				gtk_widget_hide(get_widget("check_button_use_arrow"));
			}
			gtk_widget_show(get_widget("check_button_antialias"));
			gtk_widget_show(get_widget("label_line_width"));
			gtk_widget_show(get_widget("spinbutton_line_width"));
			break;
		case 1:
		case 3:
			gtk_widget_show(get_widget("check_button_antialias"));
			gtk_widget_hide(get_widget("label_line_width"));
			gtk_widget_hide(get_widget("spinbutton_line_width"));
			break;
		case 4:
			gtk_widget_hide(get_widget("check_button_antialias"));
			gtk_widget_hide(get_widget("label_line_width"));
			gtk_widget_hide(get_widget("spinbutton_line_width"));
			break;
		case 5:
			gtk_widget_hide(get_widget("check_button_antialias"));
			gtk_widget_hide(get_widget("label_line_width"));
			gtk_widget_hide(get_widget("spinbutton_line_width"));
			break;
		default:
			gtk_widget_show(get_widget("check_button_antialias"));
			gtk_widget_show(get_widget("label_line_width"));
			gtk_widget_show(get_widget("spinbutton_line_width"));
			break;
	}
}

gchar *text_view_get_text(GtkTextView *textview, gboolean hidden_chars)
{
	GtkTextIter start, end;
	GtkTextBuffer *buffer;
	gchar *text = NULL;

	g_return_val_if_fail(GTK_IS_TEXT_VIEW(textview), NULL);

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);
	text = gtk_text_buffer_get_text(buffer, &start, &end, hidden_chars);

	return text;
}

void message_box_ok(GtkMessageType type, gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (GTK_WINDOW(gtk_widget_get_toplevel(dibuja_get_canvas())),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 type, GTK_BUTTONS_OK, "%s", buffer);

	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);
}

gint message_box_yes_no(gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog;
	gint ret;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (GTK_WINDOW(gtk_widget_get_toplevel(dibuja_get_canvas())),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_QUESTION,
                                 GTK_BUTTONS_YES_NO, "%s", buffer);

	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
	ret = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);

	return ret;
}
// Takes care of ugly selected text in dlg
void msg_dlg_forall(GtkWidget *widget, gpointer data)
{
	char *name = NULL;

	name = (char *)gtk_widget_get_name(widget);

	if(0 == g_strcmp0(name, "GtkHBox")){
		gtk_container_forall(GTK_CONTAINER(widget), msg_dlg_forall, NULL);
	}
	else if(0 == g_strcmp0(name, "GtkVBox")){
		gtk_container_forall(GTK_CONTAINER(widget), msg_dlg_forall, NULL);
	}
	else if(0 == g_strcmp0(name, "GtkLabel")){
		gtk_widget_set_can_focus(widget, FALSE);
		gtk_widget_queue_draw(widget);
	}
}

void message_box_error(gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog, *vbox;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (GTK_WINDOW(gtk_widget_get_toplevel(dibuja_get_canvas())),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_ERROR,
                                 GTK_BUTTONS_NONE, "%s", buffer);

	vbox = gtk_dialog_get_content_area(GTK_DIALOG (dialog));
	gtk_container_forall(GTK_CONTAINER(vbox), msg_dlg_forall, NULL);
	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);
}
// "Close without saving", "cancel", "save"
gint message_box_yes_no_cancel(gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog;
	gint ret;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (GTK_WINDOW(gtk_widget_get_toplevel(dibuja_get_canvas())),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_QUESTION,
                                 GTK_BUTTONS_YES_NO, "%s", buffer);

	gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
	ret = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);

	return ret;
}
/*
void rect_from_pts(int p1x, int p1y, int p2x, int p2y, GdkRectangle *rectangle)
{
	if(!rectangle) return;

	rectangle->x = MIN(p1x, p2x);
	rectangle->y = MIN(p1y, p2y);
	rectangle->width = ABS(p1x - p2x);
	rectangle->height = ABS(p1y - p2y);
}

void resize_rect(GdkRectangle *rectangle, int x, int y)
{
	if(!rectangle) return;

	rectangle->x -= x;
	rectangle->y -= y;
	rectangle->width += y * 2;
	rectangle->height += y * 2;
}
*/
/*void set_update_rect_orig(GdkRectangle *r, int p1x, int p1y, int p2x, int p2y, double z)
{

	if(r){
		r->x = (int)img_to_win((double)(MIN(p1x, p2x) - 2), z);
		r->y = (int)img_to_win((double)(MIN(p1y, p2y) - 2), z);
		r->width = (int)img_to_win((double)(ABS(p1x - p2x) + 4), z);
		r->height = (int)img_to_win((double)(ABS(p1y - p2y) + 4), z);
		resize_rect(r, 4, 4);
	}
}*/

void set_update_rect(GdkRectangle *r, int p1x, int p1y, int p2x, int p2y, double z)
{

	if(r){
		r->x = (int)img_to_win((double)(MIN(p1x, p2x)), z);
		r->y = (int)img_to_win((double)(MIN(p1y, p2y)), z);
		r->width = (int)img_to_win((double)(ABS(p1x - p2x)), z);
		r->height = (int)img_to_win((double)(ABS(p1y - p2y)), z);
		//resize_rect(r, 4, 4);
	}
}
/*
void debug_rect(CR_TYPE type, GdkRectangle *r)
{
	cairo_t *cr;
	
	if(!r) return;
	cr = dibuja_get_cr(type);
	cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
	cairo_set_line_width(cr, 1.0);
	gdk_cairo_rectangle(cr, r);
	cairo_stroke(cr);
	cairo_destroy(cr);
}*/
/*
void debug_rect_from_pts(CR_TYPE type, int p1x, int p1y, int p2x, int p2y)
{
	GdkRectangle r;

	rect_from_pts(p1x, p1y, p2x, p2y, &r);
	debug_rect(type, &r);
}
*/
void canvas_set_wait_cursor(GtkWidget *widget)
{
	GdkCursor *cur;

	if(!GDK_IS_WINDOW (widget->window)) return;

	cur = gdk_cursor_new_for_display(gdk_display_get_default(), GDK_WATCH);
	if(cur)
		gdk_window_set_cursor(widget->window, cur);
}

GdkCursor *canvas_load_cursor(GdkCursorType type)
{
	GdkPixbuf *pixbuf = NULL;
	GdkCursor *cur = NULL;

	pixbuf = file_load_pixbuf(DIBUJA_CURSOR_DIR	"dibuja-crosshair-cursor.png", FALSE);

	if(!GDK_IS_PIXBUF(pixbuf))
		return gdk_cursor_new_for_display(gdk_display_get_default(), GDK_LEFT_PTR);

	cur = gdk_cursor_new_from_pixbuf(gdk_display_get_default(), pixbuf, 10, 10);
	return cur;
}

void canvas_set_cursor(GtkWidget *widget, GdkCursorType type)
{
	GdkCursor *cur;
	
	if(!GDK_IS_WINDOW (widget->window)) return;
	if((DIBUJA_CURSOR != type) && !(type >= GDK_X_CURSOR && type < GDK_LAST_CURSOR)) return;

	if(DIBUJA_CURSOR == type){
		cur = canvas_load_cursor(type);
	}
	else{
		cur = gdk_cursor_new_for_display(gdk_display_get_default(), type);
	}

	if(cur)
		gdk_window_set_cursor(widget->window, cur);
	else{
		g_message("Error creating cursor\n");
	}
}

/***************************************************************************/

/**
 * gimp_get_screen_resolution:
 * @screen: a #GdkScreen or %NULL
 * @xres: returns the horizontal screen resolution (in dpi)
 * @yres: returns the vertical screen resolution (in dpi)
 *
 * Retrieves the screen resolution from GDK. If @screen is %NULL, the
 * default screen is used.
 **/
void
gimp_get_screen_resolution (GdkScreen *screen,
                            gdouble   *xres,
                            gdouble   *yres)
{
  gint    width, height;
  gint    width_mm, height_mm;
  gdouble x = 0.0;
  gdouble y = 0.0;

  g_return_if_fail (screen == NULL || GDK_IS_SCREEN (screen));
  g_return_if_fail (xres != NULL);
  g_return_if_fail (yres != NULL);

  if (!screen)
    screen = gdk_screen_get_default ();

  width  = gdk_screen_get_width (screen);
  height = gdk_screen_get_height (screen);

  width_mm  = gdk_screen_get_width_mm (screen);
  height_mm = gdk_screen_get_height_mm (screen);

  /*
   * From xdpyinfo.c:
   *
   * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
   *
   *     dpi = N pixels / (M millimeters / (25.4 millimeters / 1 inch))
   *         = N pixels / (M inch / 25.4)
   *         = N * 25.4 pixels / M inch
   */

  if (width_mm > 0 && height_mm > 0)
    {
      x = (width  * 25.4) / (gdouble) width_mm;
      y = (height * 25.4) / (gdouble) height_mm;
    }

  if (x < GIMP_MIN_RESOLUTION || x > GIMP_MAX_RESOLUTION ||
      y < GIMP_MIN_RESOLUTION || y > GIMP_MAX_RESOLUTION)
    {
      g_warning ("GDK returned bogus values for the screen resolution, "
                 "using 96 dpi instead.");

      x = 96.0;
      y = 96.0;
    }

  /*  round the value to full integers to give more pleasant results  */
  *xres = ROUND (x);
  *yres = ROUND (y);
}

void util_spin_set_val(char *adj, gdouble value)
{
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget(adj)), value);
}
/***************************************************************************/

static void save_widget(char *wid, char *as)
{
	GtkWidget *widget;

	widget = GTK_WIDGET (gtk_builder_get_object (gtk_builder, wid));
	if(!GTK_IS_WIDGET(widget)) {
		g_warning("Could not save widget: ");
		g_warning("    \"%s\" as \"%s\"", wid, as);
		return;
	}
	g_object_set_data(G_OBJECT(main_window), as, (gpointer)widget);
}

void util_save_widgets(GtkWidget *window, GtkBuilder *builder)
{
	GtkWidget *dialog, *textview, *button, *widget,
			  *menu, *menu_item;

	main_window = window;
	gtk_builder = builder;

// Tools
	save_widget("check_button_transparent_selection",
				"check_button_transparent_selection");
	save_widget("check_button_antialias", "check_button_antialias");
	save_widget("check_button_use_arrow", "check_button_use_arrow");
	save_widget("label_line_width", "label_line_width");
	save_widget("spinbutton_line_width", "spinbutton_line_width");
	save_widget("table1", "table1");
	save_widget("spinbutton_brush_width", "spinbutton_brush_width");
	save_widget("spinbutton_brush_spacing", "spinbutton_brush_spacing");
	save_widget("Tool Properties", "Tool Properties");
	save_widget("tb_apply_selection", "tb_apply_selection");
	save_widget("tb_clear_selection", "tb_clear_selection");

// Colors
	save_widget("table_col_pal", "table_col_pal");
	save_widget("colorbutton_fg", "colorbutton_fg");
	save_widget("colorbutton_bg", "colorbutton_bg");

// Statusbar
	save_widget("statusbar1", "statusbar1");

	button = GTK_WIDGET (gtk_builder_get_object (builder, "colorbutton_fg"));
	g_object_set_data(G_OBJECT(window), "colorbutton_fg", (gpointer)button);
	button = GTK_WIDGET (gtk_builder_get_object (builder, "colorbutton_bg"));
	g_object_set_data(G_OBJECT(window), "colorbutton_bg", (gpointer)button);

// Toolbar zoom combo & buttons
	save_widget("toolbar1", "toolbar1");
	save_widget("tb_zoom_in", "tb_zoom_in");
	save_widget("tb_zoom_out", "tb_zoom_out");
	save_widget("tb_cut", "tb_cut");

// Toolbar undo & redo buttons
	button = GTK_WIDGET (gtk_builder_get_object (builder, "tb_undo"));
	gtk_widget_set_sensitive(button, FALSE);
	g_object_set_data(G_OBJECT(window), "tb_undo", (gpointer)button);
	button = GTK_WIDGET (gtk_builder_get_object (builder, "tb_redo"));
	gtk_widget_set_sensitive(button, FALSE);
	g_object_set_data(G_OBJECT(window), "tb_redo", (gpointer)button);

// Text tool TODO: Nicer text tool
	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog-text"));
	g_object_set_data(G_OBJECT(window), "text-dialog", (gpointer)dialog);

	button = GTK_WIDGET (gtk_builder_get_object (builder, "fontbutton1"));
	g_object_set_data(G_OBJECT(window), "fontbutton", (gpointer)button);

	textview = GTK_WIDGET (gtk_builder_get_object (builder, "textview1"));
	g_object_set_data(G_OBJECT(window), "textview", (gpointer)textview);

	widget = GTK_WIDGET (gtk_builder_get_object (builder, "check_box_use_fill_prop"));
	g_object_set_data(G_OBJECT(window), "use_fill_prop", (gpointer)widget);

	widget = GTK_WIDGET (gtk_builder_get_object (builder, "check_button_center_horz"));
	g_object_set_data(G_OBJECT(window), "center_horz", (gpointer)widget);

	widget = GTK_WIDGET (gtk_builder_get_object (builder, "check_button_center_vert"));
	g_object_set_data(G_OBJECT(window), "center_vert", (gpointer)widget);

	save_widget("check_button_meme_text", "meme-text");

// Edit Menu
	save_widget("menu2", "menu2");
	menu = GTK_WIDGET (gtk_builder_get_object (builder, "menu_edit"));
	g_object_set_data(G_OBJECT(window), "editmenu", menu);

	menu_item = GTK_WIDGET (gtk_builder_get_object (builder, "menu_edit_undo"));
	g_object_set_data(G_OBJECT(window), "edit_undo", menu_item);
	gtk_widget_set_sensitive(menu_item, FALSE);

	menu_item = GTK_WIDGET (gtk_builder_get_object (builder, "menu_edit_redo"));
	g_object_set_data(G_OBJECT(window), "edit_redo", menu_item);
	gtk_widget_set_sensitive(menu_item, FALSE);

	menu_item = GTK_WIDGET (gtk_builder_get_object (builder, "menu_edit_paste"));
	g_object_set_data(G_OBJECT(window), "edit_paste", menu_item);

	menu_item = GTK_WIDGET (gtk_builder_get_object (builder, "menu_edit_copy"));
	g_object_set_data(G_OBJECT(window), "edit_copy", menu_item);

	save_widget("menu_edit_cut", "menu_edit_cut");

// Effects Menu
	save_widget("menu_effects_pixelate", "menu_effects_pixelate");

// Image Menu
	save_widget("menu_image_crop", "menu_image_crop");
	save_widget("menu_image_selection_as_brush",
				"menu_image_selection_as_brush");

// New File Dialog
	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_new_file_at_size"));
	g_object_set_data(G_OBJECT(window), "dialog_new_image", dialog);
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "spin_new_image_width"));
	g_object_set_data(G_OBJECT(window), "new_image_width_spin", widget);
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "spin_new_image_height"));
	g_object_set_data(G_OBJECT(window), "new_image_height_spin", widget);
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combo_filesize_type"));
	g_object_set_data(G_OBJECT(window), "filesize_type_combo", widget);

// Color Edit Dialog
	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "color_sel_edit_colors"));
	g_object_set_data(G_OBJECT(window), "color_edit_dlg", dialog);

// Rotate dialog
	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_rotate"));
	g_object_set_data(G_OBJECT(window), "dialog_rotate", dialog);
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_rotate_darea"));
	g_object_set_data(G_OBJECT(window), "dialog_rotate_darea", widget);
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_rotate_angle_spin"));
	g_object_set_data(G_OBJECT(window), "dialog_rotate_angle_spin", widget);

// Resize Dialog
	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_resize"));
	g_object_set_data(G_OBJECT(window), "dialog_resize", dialog);

	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_resize_entry_width"));
	g_object_set_data(G_OBJECT(window), "dialog_resize_entry_width", widget);
	
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_resize_entry_height"));
	g_object_set_data(G_OBJECT(window), "dialog_resize_entry_height", widget);
	
	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_resize_label_size"));
	g_object_set_data(G_OBJECT(window), "dialog_resize_label_size", widget);

	save_widget("dialog_resize_checkbutton", "dialog_resize_checkbutton");

// Scale Dialog
	save_widget("dialog_scale", "dialog_scale");
	save_widget("dialog_scale_label_size", "dialog_scale_label_size");
	save_widget("dialog_scale_spinbutton_width", "dialog_scale_spinbutton_width");
	save_widget("dialog_scale_spinbutton_height", "dialog_scale_spinbutton_height");


	save_widget("scrolledwindow1", "scroll");
	gtk_builder = NULL;
}

char *util_get_config_filename(void)
{
	char *filename;
	char *config_dir = util_get_dibuja_config_dir();

	filename = g_build_filename(config_dir, DIBUJA_CONFIG_FILE, NULL);
	g_free(config_dir);

//printf("[%s - %s(): %d] %s\n", __FILE__, __func__, __LINE__, filename);
	return filename;
}

char *util_get_dibuja_config_dir(void)
{
	char *filename;
#ifndef WORKING_DIR
	char *config_dir = (char *)g_get_user_config_dir();
	filename = g_build_filename(config_dir, PACKAGE, NULL);
#else
	filename = g_strdup(WORKING_DIR);
#endif
//printf("[%s - %s(): %d] %s\n", __FILE__, __func__, __LINE__, filename);
	return filename;
}

GtkWidget *util_create_label(const gchar *format, ...)
{
	GtkWidget *label;
	char *markup;
	va_list args;

	va_start(args, format);
	markup = g_markup_vprintf_escaped(format, args);
	va_end(args);

	label = gtk_label_new(NULL);
	if(markup){
		gtk_label_set_markup(GTK_LABEL(label), markup);
		g_free(markup);
	}

	return label;
}


